#ifndef AbstractGraph_hpp
#define AbstractGraph_hpp

#ifdef WIN32
typedef __int64 int64_t;
#else
#include <stdint.h>
#endif
#include <string>

#include <QVector3D>

#include "precision.hpp"

class AbstractGraph {	
public:	
	AbstractGraph(QVector3D offset);	// offset in OpenGL units
	virtual ~AbstractGraph();
	
	// width: the width of the screen
	// height: the height of the screen
    virtual void draw(QMatrix4x4 & view, QMatrix4x4 & projection, size_t width, size_t height, fPoint maxValue, fPoint minValue, fPoint leftValue, fPoint rightValue, bool rotated, float zoomFactor) = 0;
	virtual const QVector3D * getDataPtr() const = 0;
	virtual fPoint getMaxValue() const = 0;	// returns the maximum y value
	virtual fPoint getMinValue() const = 0;	// returns the minimum y value
	virtual std::string getDescription() const = 0;
	virtual size_t getSize() const = 0;	// the number of values
	virtual QVector3D getOffset() const;
	
protected:
	QVector3D offset_;	// the offset for drawing
};

// functor class for sorting AbstractGraph objects
class GraphComparator {
public:
	bool operator()(const AbstractGraph * g1, const AbstractGraph * g2) const
	{
		QVector3D o1 = g1->getOffset();
		QVector3D o2 = g2->getOffset();
		if (o1.z() == o2.z()) {
			return g1 < g2;	// so we can have multiple graphs at the same depth
		}
		return (o1.z() < o2.z());
	}
};

#endif