#include "StackGrapher.hpp"
#include <iostream>
#include <cstdlib>
#include <QtGui>
#include <QFileDialog>
#include <QMessageBox>
#include <cmath>
#include <sstream>
#include <numeric>
#include "Grapher.hpp"

StackGrapher::StackGrapher(QWidget *parent) : QMainWindow(parent),
	frame(0)
{
	setupUi();
	
	loadData(":/icons/testData.csv");
	
	//timer = new QTimer();
	//timer->setSingleShot(false);
	//connect(timer, SIGNAL(timeout()), this, SLOT(update()));
	//timer->start(60);
	
	//movetimer = new QTimer();
	//connect(movetimer, SIGNAL(timeout()), this, SLOT(movegraph()));
	//movetimer->start(140);
	
	//grapher->center(50);
	
	//connect(grapher, SIGNAL(centerChanged(size_t)), this, SLOT(centered(size_t)));

	//connect( ui->saveButton, SIGNAL(clicked()), this, SLOT(save()));
}

StackGrapher::~StackGrapher()
{
}

void StackGrapher::redraw()
{
	grapher->update();
}

void StackGrapher::movegraph()
{
	grapher->center(++frame);
}

void StackGrapher::centered(size_t value)
{
}

void StackGrapher::bigger()
{
	int size = grapher->getFontSize();
	grapher->setFontSize(++size);
}

void StackGrapher::smaller()
{
	int size = grapher->getFontSize();
	size--;
	
	if (size > 0) {
		grapher->setFontSize(size);
	}
}

void StackGrapher::setupUi()
{
	resize(685, 557);

	QAction* actionOpen = new QAction(QIcon(":/icons/open.png"), tr("Open CSV File..."), this);
	actionOpen->setObjectName(QString::fromUtf8("actionOpen"));

	QAction* actionSave = new QAction(QIcon(":/icons/save.png"), tr("Save Image As..."), this);
	actionSave->setObjectName(QString::fromUtf8("actionSave"));

	QAction* actionAntiAliasing = new QAction(tr("Anti-Aliasing"), this);
	actionAntiAliasing->setObjectName(QString::fromUtf8("actionAntiAliasing"));
	actionAntiAliasing->setCheckable(true);

	QAction* actionRotate = new QAction(tr("Rotate Graph"), this);
	actionRotate->setObjectName(QString::fromUtf8("actionRotate"));
	actionRotate->setCheckable(true);

	QAction* actionBigger = new QAction(QIcon(":/icons/plus.png"), tr("Increase Font Size"), this);
	actionBigger->setObjectName(QString::fromUtf8("actionBigger"));

	QAction* actionSmaller = new QAction(QIcon(":/icons/minus.png"), tr("Decrease Font Size"), this);
	actionSmaller->setObjectName(QString::fromUtf8("actionSmaller"));

	QAction* actionAuto_Scale = new QAction(tr("Auto-Scale"), this);
	actionAuto_Scale->setObjectName(QString::fromUtf8("actionAuto_Scale"));
	actionAuto_Scale->setCheckable(true);

	QAction* actionAuto_Hover = new QAction(tr("Auto-Hover"), this);
	actionAuto_Hover->setObjectName(QString::fromUtf8("actionAuto_Hover"));
	actionAuto_Hover->setCheckable(true);

	QWidget* centralWidget = new QWidget(this);
	QHBoxLayout* horizontalLayout = new QHBoxLayout(centralWidget);

	grapher = new Grapher("time");
	QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	sizePolicy.setHorizontalStretch(0);
	sizePolicy.setVerticalStretch(0);
	sizePolicy.setHeightForWidth(grapher->sizePolicy().hasHeightForWidth());
	grapher->setSizePolicy(sizePolicy);
	grapher->setMinimumSize(QSize(657, 473));

	horizontalLayout->addWidget(grapher);

	setCentralWidget(centralWidget);

	QMenuBar* menuBar = new QMenuBar(this);
	QMenu* menuFile = new QMenu(menuBar);
	QMenu* menuDisplay = new QMenu(menuBar);
	setMenuBar(menuBar);

	menuBar->addAction(menuFile->menuAction());
	menuBar->addAction(menuDisplay->menuAction());

	menuFile->addAction(actionOpen);
	menuFile->addAction(actionSave);

	menuDisplay->addAction(actionAntiAliasing);
	menuDisplay->addAction(actionRotate);
	menuDisplay->addAction(actionBigger);
	menuDisplay->addAction(actionSmaller);
	menuDisplay->addAction(actionAuto_Scale);
//	menuDisplay->addAction(actionAuto_Hover);

	// from retranslateUi
	setWindowTitle(QApplication::translate("StackGrapher", "StackGrapher", 0, QApplication::UnicodeUTF8));
	menuFile->setTitle(QApplication::translate("StackGrapher", "File", 0, QApplication::UnicodeUTF8));
	menuDisplay->setTitle(QApplication::translate("StackGrapher", "Display", 0, QApplication::UnicodeUTF8));

	QToolBar* fileToolBar = addToolBar(tr("File"));
	fileToolBar->addAction(actionOpen);
	fileToolBar->addAction(actionSave);

	QToolBar* displayToolBar = addToolBar(tr("Display"));
//	displayToolBar->addAction(actionAntiAliasing);
//	displayToolBar->addAction(actionRotate);
	displayToolBar->addAction(actionBigger);
	displayToolBar->addAction(actionSmaller);
//	displayToolBar->addAction(actionAuto_Scale);
//	displayToolBar->addAction(actionAuto_Hover);

/*	statusBar = new QStatusBar(this);
	setStatusBar(statusBar);
*/

	QMetaObject::connectSlotsByName(this);
}
/*
void StackGrapher::generateTestData()
{
	const size_t numVals = 1000;
	const size_t graphCount = 9;

	std::vector<std::vector<fPoint> > values;
	std::vector<std::string> descriptions;

	for (size_t graphNumber = 0; graphNumber != graphCount; ++graphNumber) {
		values.resize(values.size() + 1);
		values.back().resize(numVals);
		
		srand(graphNumber);
		float add = 0;
		for (size_t i = 0; i != numVals; ++i) {
			values.back()[i] = std::abs(std::sin((double)i / 10 + graphNumber) * graphNumber);
			add += ((float)rand() / RAND_MAX - 0.5) / 10;
			values.back()[i] += 3 * std::abs(add);
		}
	}

	setData(values, descriptions);
}
*/
void StackGrapher::loadData(const QString& fileName)
{
	QFile csvFile(fileName);
	if (!csvFile.open(QFile::ReadOnly)) {
		QMessageBox::warning(this, tr("Loading CSV File"), QString("Could not be open ") + fileName + QString(" for reading."));
		return;
	}

	grapher->clear();

	std::vector<std::vector<fPoint> > values;
	std::vector<std::string> descriptions;

	QTextStream textStream(&csvFile);
	for (QString line = textStream.readLine(); !line.isNull(); line = textStream.readLine()) {
		QStringList valueStrings = line.split(',');
		if (valueStrings.size() > 1) {
			QString description = valueStrings.front();
			descriptions.push_back(description.toStdString());
			values.resize(values.size() + 1);
			values.back().reserve(valueStrings.size());
			for (size_t i = 1; i < valueStrings.size(); ++i) {
				values.back().push_back(valueStrings[i].toFloat());
			}
		}
	}

	const size_t graphCount = values.size();
	size_t numVals = 0;
	for (size_t graphNumber = 0; graphNumber != graphCount; ++graphNumber) {
		numVals = std::max(numVals, values[graphNumber].size());
	}

	std::vector<fPoint> baseLine(numVals);
	for (size_t graphNumber = 0; graphNumber != graphCount; ++graphNumber) {
		for (size_t i = 0; i != numVals; ++i) {
			if (i < values[graphNumber].size()) {
				baseLine[i] -= 1 / ((float)graphCount + 1) * ((float)graphCount - (float)graphNumber) * values[graphNumber][i];
			}
		}
	}

	// calculate relative graph areas (to set the color saturation accordingly)
	std::vector<double> relativeAreas(graphCount);
	for (size_t graphNumber = 0; graphNumber != graphCount; ++graphNumber) {
		for (size_t i = 0; i != values[graphNumber].size(); ++i) {
			relativeAreas[graphNumber] += static_cast<double>(values[graphNumber][i]);
		}
	}
	double maxArea = 0.0;
	for (size_t graphNumber = 0; graphNumber != graphCount; ++graphNumber) {
		if (relativeAreas[graphNumber] > maxArea) {
			maxArea = relativeAreas[graphNumber];
		}
	}
	if (maxArea != 0.0) {
		for (size_t graphNumber = 0; graphNumber != graphCount; ++graphNumber) {
			relativeAreas[graphNumber] /= maxArea;
		}
	}

	std::vector<std::vector<fPoint> > bottom(graphCount, std::vector<fPoint>(numVals));
	std::vector<std::vector<fPoint> > top(graphCount, std::vector<fPoint>(numVals));

	for (size_t graphNumber = 0; graphNumber != bottom.size(); ++graphNumber) {
		if (graphNumber == 0) {
			bottom[graphNumber] = baseLine;
		} else {
			bottom[graphNumber] = top[graphNumber - 1];
		}

		for (size_t i = 0; i != numVals; ++i) {
			if (i < values[graphNumber].size()) {
				top[graphNumber][i] = bottom[graphNumber][i] + values[graphNumber][i];
			} else {
				top[graphNumber][i] = bottom[graphNumber][i];
			}
		}

		QColor color = QColor::fromHsvF(graphNumber / (bottom.size() * 1.5), 1.0, 0.8);
		QVector4D colorVector(color.redF(), color.greenF(), color.blueF(), 1.0);
//		QVector4D colorVector((float)rand() / RAND_MAX, (float)rand() / RAND_MAX, (float)rand() / RAND_MAX, 1.0);

		std::string description = graphNumber < descriptions.size() ? descriptions[graphNumber] : std::string();
		grapher->addStackedGraph(bottom[graphNumber].begin(), bottom[graphNumber].end(), top[graphNumber].begin(), colorVector, 0.75, 1, QVector3D(0.0, 0.0, graphNumber), description);
	}

	grapher->addUnitGraph(1000, 1.0f, true, QVector4D(0.0, 0.0, 0.0, 1.0));

	setWindowTitle(fileName + " - StackGrapher");
}

void StackGrapher::on_actionOpen_triggered()
{
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open CSV File"), ".", tr("CSV Files (*.csv)"));
	if (fileName.isEmpty()) {
		return;
	}
	loadData(fileName);
}

void StackGrapher::on_actionSave_triggered()
{
	QString fileName = QFileDialog::getSaveFileName(this, tr("Save Image"), ".", tr("Image Files (*.png *.jpg)"));
	if (fileName.isEmpty()) {
		return;
	}
	grapher->saveTo(fileName);
}

void StackGrapher::on_actionBigger_triggered()
{
	bigger();
}

void StackGrapher::on_actionSmaller_triggered()
{
	smaller();
}

void StackGrapher::on_actionAntiAliasing_triggered(bool checked)
{
	grapher->setAntiAliasing(checked);
}

void StackGrapher::on_actionRotate_triggered(bool checked)
{
	grapher->setRotated(checked);
}

void StackGrapher::on_actionAuto_Scale_triggered(bool checked)
{
	grapher->setAutoScale(checked);
}

void StackGrapher::on_actionAuto_Hover_triggered(bool checked)
{
	grapher->setAutoHover(checked);
}
