#ifndef StackedGraph_hpp
#define StackedGraph_hpp

#include "ContinuousGraph.hpp"

class StackedGraph : public ContinuousGraph {
public:
	template<class DataIterator>
	StackedGraph(DataIterator dataBegin, DataIterator dataEnd, DataIterator topBegin, QVector4D color = QVector4D(0.0, 0.0, 0.0, 1.0), float fillAlpha = 0.9, float lineWidth = 1, QVector3D offset = QVector3D(0.0, 0.0, 0.0), const std::string& unit = std::string()) :
		ContinuousGraph(dataBegin, dataEnd, color, lineWidth, offset, unit),
		fillAlpha_(fillAlpha)
	{
		convert(dataBegin, dataEnd, topBegin, &color, 1);
		calculateExtremeValues();
		generateIndicesAndRegenerateVertexBuffer();
	}

	template<class DataIterator, class ColorIterator>
	StackedGraph(DataIterator dataBegin, DataIterator dataEnd, DataIterator topBegin, ColorIterator colorsBegin, float fillAlpha = 0.9, float lineWidth = 1, QVector3D offset = QVector3D(0.0, 0.0, 0.0), const std::string& unit = std::string()) :
		ContinuousGraph(dataBegin, dataEnd, colorsBegin, lineWidth, offset, unit),
		fillAlpha_(fillAlpha)
	{
		convert(dataBegin, dataEnd, topBegin, colorsBegin, dataEnd - dataBegin);
		calculateExtremeValues();
		generateIndicesAndRegenerateVertexBuffer();
	}
	
    virtual void draw(QMatrix4x4 & view, QMatrix4x4 & projection, size_t width, size_t height, fPoint maxValue, fPoint minValue, fPoint leftValue, fPoint rightValue, bool rotated, float zoomFactor);
		
	virtual void generateIndicesAndRegenerateVertexBuffer();

	QVector3D getBottom(size_t i) const;
	QVector3D getTop(size_t i) const;

private:
	template<class DataIterator, class ColorIterator>
	void convert(DataIterator dataBegin, DataIterator dataEnd, DataIterator topBegin, ColorIterator colorsBegin, size_t numColors)
	{
		size_t numValues = dataEnd - dataBegin;
		positions_.resize(numValues);
		top_.resize(numValues);
		colors_.resize(numValues);
		
		for (size_t i = 0; i < numValues; ++i) {
			positions_[i] = QVector3D((float) i, dataBegin[i], 0.0f);
			top_[i] = QVector3D((float) i, topBegin[i], 0.0f);
			if (i < numColors) {
				colors_[i] = colorsBegin[i];
			} else {
				colors_[i] = colorsBegin[numColors - 1];
			}
		}
	}

	void calculateExtremeValues();

	std::vector<QVector3D> top_;
	float fillAlpha_;
};

#endif
