#ifndef UnitGraph_hpp
#define UnitGraph_hpp

#include "ContinuousGraph.hpp"

// a class providing the functionality to draw units with LoD
// units are drawn in the y range [-1, 1], so the projection has to be adjusted
class UnitGraph : public ContinuousGraph {
public:
	
	// numValues: the number of unit values
	// spacing: the frequency of the units, eg. unit every minValue + n * spacing
	// repeat: if the units have to be repeated
	UnitGraph(size_t numValues, fPoint spacing, bool repeat, QVector4D color = QVector4D(0.0, 0.0, 0.0, 1.0), float lineWidth = 1, QVector3D offset = QVector3D(0.0, 0.0, 0.0), const std::string& unit = std::string());
	
    virtual void draw(QMatrix4x4 & view, QMatrix4x4 & projection, size_t width, size_t height, fPoint maxValue, fPoint minValue, fPoint leftValue, fPoint rightValue, bool rotated, float zoomFactor);
	
	size_t getNumberOfValues() const;	// returns the number of grid values
		
	virtual void generateIndicesAndRegenerateVertexBuffer();

private:
	size_t numValues_;	// the number of grid values
	fPoint spacing_;
	bool repeat_;	// if it will be repeated
	QVector4D color_;
	std::vector<int> spacings_;	// the spacings of the levels
};

#endif
