#ifndef VertexBuffer_hpp
#define VertexBuffer_hpp

#include <QGLWidget>
#include <QVector2D>
#include <QVector3D>
#include <QVector4D>
#include <QGLBuffer>
#include <vector>

class VertexBuffer {
public:
	VertexBuffer();
	VertexBuffer(const std::vector<int> & indices, const std::vector<QVector3D> & positions);
	VertexBuffer(const std::vector<int> & indices, const std::vector<QVector3D> & positions, const std::vector<QVector4D> &colors);
	VertexBuffer(const std::vector<int> & indices, const std::vector<QVector3D> & positions, const std::vector<QVector2D> &texCoords);
	~VertexBuffer();
	
	void bind();
	void release();
	
	float getXMin() const;
	void setXMin(float m_Xmin);
	float getXMax() const;
	void setXMax(float m_Xax);
	
	size_t getNumberOfIndices() const;
	
	void setBufferOffset(QVector3D & bufferOffset);
	const QVector3D & getBufferOffset() const;
	
private:
	void upload();
	void clear();
		
	std::vector<int> indices_;
	QGLBuffer* indexBuffer_;
	
	std::vector<QVector3D> positions_;	// the vertices stored on the CPU. Data only having y coordinates will be transformed to have (x,y,z).
	QGLBuffer* positionBuffer_;
	
	std::vector<QVector4D> colors_;	// the colors of each vertex
	QGLBuffer* colorBuffer_;
	
	std::vector<QVector2D> texCoords_;
	QGLBuffer* texCoordBuffer_;
	
	float xMin_;
	float xMax_;
	
	QVector3D bufferOffset_;	// the offset of the buffers coordinates in world coordinates
	
};

#endif
